﻿/*---------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*----------------------------------------------------------------------------
* Class FormMain.
* Provides the interaction with the demo user interface,
* uses CMomanLibSample to get access to the FAULHABER communiction API.
*----------------------------------------------------------------------------
*/
using System;
using System.Windows.Forms;
using System.Threading;
using System.Threading.Tasks;

namespace DemoMomanLib
{

    public partial class FormMain : Form
    {
        int cNodeNr = 1;    //node number of the connected device
        
        CMomanLibSample momanLibSample;
        private bool m_bMomanLibLoaded;
        Thread ReceiveThread;

        public FormMain()
        {
            InitializeComponent();
            momanLibSample = new CMomanLibSample();
            if (momanLibSample.Init(CBAsyncDataReceived) == true) {
                m_bMomanLibLoaded = true;
                this.statusStrip1.Items[0].Text = "FAULHABER communication API loaded";
				btnGetStrObj.Enabled = true;
				btnShutdown.Enabled = true;
				btnSwitchOn.Enabled = true;
				btnEnOp.Enabled = true;				
            }
            else {
                m_bMomanLibLoaded = false;
                this.statusStrip1.Items[0].Text = "";
                MessageBox.Show("Interface can not be opened!");
            }
        }
        ~FormMain()
        { 
        }

        private AutoResetEvent ReceiveEvent = new AutoResetEvent(false);
        private void CBAsyncDataReceived()
        {
            ReceiveEvent.Set();
        }
        private void EventReceiver()
        {
            while (true) {
                ReceiveEvent.WaitOne();
                Invoke(new MethodInvoker(AsyncDataReceived));
                ReceiveEvent.Reset();
            }
        }
        private void AsyncDataReceived()
        {
            //Asynchronous data received
            string data, cmd;
            momanLibSample.ReadReceivedData(out data, out cmd);
            if (cmd != "") {
                AddText("Async Received: " + data + " (Cmd = " + cmd +")");
            }
            else {
                AddText("Async Received: " + data);
            }

            if ((momanLibSample.GetStatusword() & 0x00FF) == 0x0027) {	//"Operation Enabled"
                btnRelPos.Enabled = true;
            }
            else {
                btnRelPos.Enabled = false;
            }
        }

        private void AddText(string p)
        {
             this.textBox1.AppendText(p + "\r\n");
        }


        private void btnGetStrObj_Click(object sender, EventArgs e)
        {
            if (m_bMomanLibLoaded) {
		        string value;
		        //Read device name
                momanLibSample.GetStrObj(cNodeNr, 0x1008, 0x00, out value);
		        AddText("Sync Read 0x1008.00");
		        AddText("Sync Received: " + value);
	        }
        }

        private void btnShutdown_Click(object sender, EventArgs e)
        {
            if (m_bMomanLibLoaded) {
                if (momanLibSample.SendCommand(cNodeNr, eMomancmd.eMomancmd_shutdown) == true) {
                    AddText("Send SHUTDOWN");
                }
            }
        }

        private void btnSwitchOn_Click(object sender, EventArgs e)
        {
            if (m_bMomanLibLoaded) {
                if (momanLibSample.SendCommand(cNodeNr, eMomancmd.eMomancmd_switchon) == true) {
                    AddText("Send SWITCHON");
                }
            }
        }

        private void btnEnOp_Click(object sender, EventArgs e)
        {
            if (m_bMomanLibLoaded) {
                if (momanLibSample.SendCommand(cNodeNr, eMomancmd.eMomancmd_EnOp) == true) {
                    AddText("Send ENOP");
                }
            }
        }

        private void btnRelPos_Click(object sender, EventArgs e)
        {
            bool err = true;

	        AddText("Execute MOVE RELATIVE");
	        //Modes of Operation = Profile Position Mode (1):
	        if (momanLibSample.SetObj(cNodeNr, 0x6060, 0x00, 1, 1) == true) {
		        //Target Position = 1000:
		        if (momanLibSample.SetObj(cNodeNr, 0x607A, 0x00, 1000, 4) == true) {
                    //Enable Operation:
			        if (momanLibSample.SetObj(cNodeNr, 0x0000, (int)eMomancmd.eMomancmd_EnOp, 0, 0) == true) {
				        //Move relative:
				        if (momanLibSample.SetObj(cNodeNr, 0x6040, 0x00, 0x007F, 2) == true) {
					        err = false;
				        }
			        }
		        }
	        }

	        if (err) {
		        string abortMessage = momanLibSample.GetAbortMessage();
		        if (abortMessage != "") {
                    AddText("Error Received: " + abortMessage);
		        }
	        }
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            ReceiveThread = new Thread(EventReceiver);
            ReceiveThread.Start();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            ReceiveThread.Abort();
            ReceiveThread.Join();
        }

    }
}
