/*---------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*----------------------------------------------------------------------------
* Class MainForm.
* Provides the interaction with the demo user interface,
* uses CMomanLibSample to get access to the FAULHABER communiction API.
*----------------------------------------------------------------------------
*/
#pragma once
#include "MomanLibSample.h"

namespace DemoVCPP {

	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;
	using namespace System::Threading;

	HANDLE hReceiveEvent = NULL;
	bool threadTerminated = false;
	static void __stdcall CBAsyncDataReceived();
	static void EventReceiver();

	const int cNodeNr = 1;	//node number of the connected device

	/// <summary>
	/// Class MainForm
	/// </summary>
	public ref class MainForm : public System::Windows::Forms::Form
	{
	public:
		MainForm(void)
		{
			InitializeComponent();
			_instance = this;
			momanLibSample = new CMomanLibSample();
			if (momanLibSample->Init((tdmmProtDataCallback)&(CBAsyncDataReceived)) == true) {
				m_bMomanLibLoaded = true;
				toolStripStatusLabel1->Text = "FAULHABER communication API loaded";
				btnGetStrObj->Enabled = true;
				btnShutdown->Enabled = true;
				btnSwitchOn->Enabled = true;
				btnEnOp->Enabled = true;
			}
			else {
				m_bMomanLibLoaded = false;
				MessageBox::Show("Interface can not be opened!");
			}
			hReceiveEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
		}

		static MainForm^ _instance;
		void AsyncDataReceived();

	protected:
		~MainForm()
		{
			if (components)	{
				delete components;
			}
			delete this->momanLibSample;
			CloseHandle(hReceiveEvent);
		}

	private:
		/// <summary>
		/// Erforderliche Designervariable.
		/// </summary>
		System::ComponentModel::IContainer^  components;
		System::Windows::Forms::Button^  btnGetStrObj;
		System::Windows::Forms::Button^  btnShutdown;
		System::Windows::Forms::Button^  btnSwitchOn;
		System::Windows::Forms::Label^  label1;
		System::Windows::Forms::Label^  label2;
		System::Windows::Forms::Label^  label3;
		System::Windows::Forms::Label^  label4;
		System::Windows::Forms::TextBox^  tb_Commlog;
		System::Windows::Forms::StatusStrip^  statusStrip1;
		System::Windows::Forms::ToolStripStatusLabel^  toolStripStatusLabel1;
		System::Windows::Forms::Button^  btnEnOp;
		System::Windows::Forms::Button^  btnRelPos;
		
		CMomanLibSample* momanLibSample;
		bool m_bMomanLibLoaded;
		Thread^ ReceiveThread;


#pragma region Windows Form Designer generated code
		/// <summary>
		/// Erforderliche Methode fr die Designeruntersttzung.
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor gendert werden.
		/// </summary>
		void InitializeComponent(void)
		{
			this->btnGetStrObj = (gcnew System::Windows::Forms::Button());
			this->btnShutdown = (gcnew System::Windows::Forms::Button());
			this->btnSwitchOn = (gcnew System::Windows::Forms::Button());
			this->label1 = (gcnew System::Windows::Forms::Label());
			this->label2 = (gcnew System::Windows::Forms::Label());
			this->label3 = (gcnew System::Windows::Forms::Label());
			this->label4 = (gcnew System::Windows::Forms::Label());
			this->tb_Commlog = (gcnew System::Windows::Forms::TextBox());
			this->statusStrip1 = (gcnew System::Windows::Forms::StatusStrip());
			this->toolStripStatusLabel1 = (gcnew System::Windows::Forms::ToolStripStatusLabel());
			this->btnEnOp = (gcnew System::Windows::Forms::Button());
			this->btnRelPos = (gcnew System::Windows::Forms::Button());
			this->statusStrip1->SuspendLayout();
			this->SuspendLayout();
			// 
			// btnGetStrObj
			// 
			this->btnGetStrObj->Enabled = false;
			this->btnGetStrObj->Location = System::Drawing::Point(105, 26);
			this->btnGetStrObj->Name = L"btnGetStrObj";
			this->btnGetStrObj->Size = System::Drawing::Size(75, 23);
			this->btnGetStrObj->TabIndex = 0;
			this->btnGetStrObj->Text = L"GetStrObj";
			this->btnGetStrObj->UseVisualStyleBackColor = true;
			this->btnGetStrObj->Click += gcnew System::EventHandler(this, &MainForm::btnGetStrObj_Click);
			// 
			// btnShutdown
			// 
			this->btnShutdown->Enabled = false;
			this->btnShutdown->Location = System::Drawing::Point(105, 55);
			this->btnShutdown->Name = L"btnShutdown";
			this->btnShutdown->Size = System::Drawing::Size(75, 23);
			this->btnShutdown->TabIndex = 1;
			this->btnShutdown->Text = L"Shutdown";
			this->btnShutdown->UseVisualStyleBackColor = true;
			this->btnShutdown->Click += gcnew System::EventHandler(this, &MainForm::btnShutdown_Click);
			// 
			// btnSwitchOn
			// 
			this->btnSwitchOn->Enabled = false;
			this->btnSwitchOn->Location = System::Drawing::Point(186, 55);
			this->btnSwitchOn->Name = L"btnSwitchOn";
			this->btnSwitchOn->Size = System::Drawing::Size(75, 23);
			this->btnSwitchOn->TabIndex = 2;
			this->btnSwitchOn->Text = L"SwitchOn";
			this->btnSwitchOn->UseVisualStyleBackColor = true;
			this->btnSwitchOn->Click += gcnew System::EventHandler(this, &MainForm::btnSwitchOn_Click);
			// 
			// label1
			// 
			this->label1->AutoSize = true;
			this->label1->Location = System::Drawing::Point(12, 31);
			this->label1->Name = L"label1";
			this->label1->Size = System::Drawing::Size(53, 13);
			this->label1->TabIndex = 3;
			this->label1->Text = L"GetStrObj";
			// 
			// label2
			// 
			this->label2->AutoSize = true;
			this->label2->Location = System::Drawing::Point(12, 60);
			this->label2->Name = L"label2";
			this->label2->Size = System::Drawing::Size(80, 13);
			this->label2->TabIndex = 4;
			this->label2->Text = L"Async. Sample:";
			// 
			// label3
			// 
			this->label3->AutoSize = true;
			this->label3->Location = System::Drawing::Point(12, 89);
			this->label3->Name = L"label3";
			this->label3->Size = System::Drawing::Size(85, 13);
			this->label3->TabIndex = 6;
			this->label3->Text = L"Position Sample:";
			// 
			// label4
			// 
			this->label4->AutoSize = true;
			this->label4->Location = System::Drawing::Point(12, 150);
			this->label4->Name = L"label4";
			this->label4->Size = System::Drawing::Size(99, 13);
			this->label4->TabIndex = 5;
			this->label4->Text = L"Communication log:";
			// 
			// tb_Commlog
			// 
			this->tb_Commlog->AcceptsReturn = true;
			this->tb_Commlog->AcceptsTab = true;
			this->tb_Commlog->Anchor = static_cast<System::Windows::Forms::AnchorStyles>((((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom) 
				| System::Windows::Forms::AnchorStyles::Left) 
				| System::Windows::Forms::AnchorStyles::Right));
			this->tb_Commlog->Location = System::Drawing::Point(3, 169);
			this->tb_Commlog->Margin = System::Windows::Forms::Padding(0);
			this->tb_Commlog->Multiline = true;
			this->tb_Commlog->Name = L"tb_Commlog";
			this->tb_Commlog->ScrollBars = System::Windows::Forms::ScrollBars::Vertical;
			this->tb_Commlog->Size = System::Drawing::Size(526, 94);
			this->tb_Commlog->TabIndex = 7;
			// 
			// statusStrip1
			// 
			this->statusStrip1->Items->AddRange(gcnew cli::array< System::Windows::Forms::ToolStripItem^  >(1) {this->toolStripStatusLabel1});
			this->statusStrip1->Location = System::Drawing::Point(0, 263);
			this->statusStrip1->Name = L"statusStrip1";
			this->statusStrip1->Size = System::Drawing::Size(529, 22);
			this->statusStrip1->TabIndex = 8;
			this->statusStrip1->Text = L"statusStrip1";
			// 
			// toolStripStatusLabel1
			// 
			this->toolStripStatusLabel1->Name = L"toolStripStatusLabel1";
			this->toolStripStatusLabel1->Size = System::Drawing::Size(0, 17);
			// 
			// btnEnOp
			// 
			this->btnEnOp->Enabled = false;
			this->btnEnOp->Location = System::Drawing::Point(267, 55);
			this->btnEnOp->Name = L"btnEnOp";
			this->btnEnOp->Size = System::Drawing::Size(75, 23);
			this->btnEnOp->TabIndex = 9;
			this->btnEnOp->Text = L"EnOp";
			this->btnEnOp->UseVisualStyleBackColor = true;
			this->btnEnOp->Click += gcnew System::EventHandler(this, &MainForm::btnEnOp_Click);
			// 
			// btnRelPos
			// 
			this->btnRelPos->Enabled = false;
			this->btnRelPos->Location = System::Drawing::Point(105, 84);
			this->btnRelPos->Name = L"btnRelPos";
			this->btnRelPos->Size = System::Drawing::Size(75, 23);
			this->btnRelPos->TabIndex = 10;
			this->btnRelPos->Text = L"Rel. Position";
			this->btnRelPos->UseVisualStyleBackColor = true;
			this->btnRelPos->Click += gcnew System::EventHandler(this, &MainForm::btnRelPos_Click);
			// 
			// MainForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->ClientSize = System::Drawing::Size(529, 285);
			this->Controls->Add(this->btnRelPos);
			this->Controls->Add(this->btnEnOp);
			this->Controls->Add(this->statusStrip1);
			this->Controls->Add(this->tb_Commlog);
			this->Controls->Add(this->label3);
			this->Controls->Add(this->label4);
			this->Controls->Add(this->label2);
			this->Controls->Add(this->label1);
			this->Controls->Add(this->btnSwitchOn);
			this->Controls->Add(this->btnShutdown);
			this->Controls->Add(this->btnGetStrObj);
			this->Name = L"MainForm";
			this->Text = L"FAULHABER USB_Demo VCPP";
			this->FormClosed += gcnew System::Windows::Forms::FormClosedEventHandler(this, &MainForm::MainForm_FormClosed);
			this->Load += gcnew System::EventHandler(this, &MainForm::MainForm_Load);
			this->statusStrip1->ResumeLayout(false);
			this->statusStrip1->PerformLayout();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion


		private: System::Void btnGetStrObj_Click(System::Object^  sender, System::EventArgs^  e)
		{
			if (m_bMomanLibLoaded) {
				//Read device name
				std::string value;
				momanLibSample->GetStrObj(cNodeNr, 0x1008, 0x00, value);
				this->tb_Commlog->AppendText(L"Sync Read 0x1008.00\r\n");
				String^ text = gcnew String(("Sync Received: " + value + "\r\n").c_str());
				this->tb_Commlog->AppendText(text);
			}
		}

		private: System::Void btnShutdown_Click(System::Object^  sender, System::EventArgs^  e)
		{
			if (m_bMomanLibLoaded) {
				if (momanLibSample->SendCommand(cNodeNr, eMomancmd_shutdown) == true) {
					this->tb_Commlog->AppendText(L"Send SHUTDOWN\r\n");
				}
			}
		}

		private: System::Void btnSwitchOn_Click(System::Object^  sender, System::EventArgs^  e)
		{
			if (m_bMomanLibLoaded) {
				if (momanLibSample->SendCommand(cNodeNr, eMomancmd_switchon) == true) {
					this->tb_Commlog->AppendText(L"Send SWITCHON\r\n");
				}
			}
		}

		private: System::Void btnEnOp_Click(System::Object^  sender, System::EventArgs^  e)
		{
			if (m_bMomanLibLoaded) {
				if (momanLibSample->SendCommand(cNodeNr, eMomancmd_EnOp) == true) {
					this->tb_Commlog->AppendText(L"Send ENOP\r\n");
				}
			}
		}

		private: System::Void btnRelPos_Click(System::Object^  sender, System::EventArgs^  e)
		{
			bool err = true;

			this->tb_Commlog->AppendText(L"Execute MOVE RELATIVE\r\n");
			//Modes of Operation = Profile Position Mode (1):
			if (momanLibSample->SetObj(cNodeNr, 0x6060, 0x00, 1, 1) == true) {
				//Target Position = 1000:
				if (momanLibSample->SetObj(cNodeNr, 0x607A, 0x00, 1000, 4) == true) {
					//Enable Operation:
					if (momanLibSample->SetObj(cNodeNr, 0x0000, eMomancmd_EnOp, 0, 0) == true) {
						//Move relative:
						if (momanLibSample->SetObj(cNodeNr, 0x6040, 0x00, 0x007F, 2) == true) {
							err = false;
						}
					}
				}
			}

			if (err) {
				std::string abortMessage = momanLibSample->GetAbortMessage();
				String^ text = gcnew String(("Error Received: " + abortMessage + "\r\n").c_str());
				if (abortMessage != "") {
					this->tb_Commlog->AppendText(text);
				}
			}
		}

		private: System::Void MainForm_Load(System::Object^  sender, System::EventArgs^  e)
		{
			ReceiveThread = gcnew Thread(gcnew ThreadStart(&EventReceiver));
			ReceiveThread->Start();	
		}

		private: System::Void MainForm_FormClosed(System::Object^  sender, System::Windows::Forms::FormClosedEventArgs^  e)
		{
			threadTerminated = true;
			SetEvent(hReceiveEvent);	//terminate WaitForSingleObject in EventReceiver()
			ReceiveThread->Join();
		}

	};	//end class

}  //end namespace
