{*--------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*---------------------------------------------------------------------------
* class TMomanLibSample as wrapper for
* MomanLib API functions used in the delphi demo examples
*---------------------------------------------------------------------------
*}
unit MomanLibSample;

//---------------------------------------------------------------------------
interface

const
  //Used MomanLib Dlls:
  cProtDll = '..\..\..\Lib\Bin\Protocol\USB\CO_USB.dll';
  cIntfDll = '..\..\..\Lib\Bin\Interface\USB\MC3USB.dll';

type
  //From MomanLib imported definies
  tdmmProtDataCallback = procedure; stdcall;
  tdmmProtTraceValuesCallback = procedure(nodeNr: Integer; value: array of Cardinal; timecode: Integer); stdcall;
  eMomanprot = (
    eMomanprot_ok_bootup = 2,
    eMomanprot_ok_async = 1,
    eMomanprot_ok = 0,
    eMomanprot_error = -1,
    eMomanprot_error_timeout = -2,
    eMomanprot_error_cmd = -3,
    eMomanprot_error_emcy = -4,
    eMomanprot_error_param = -5,
    eMomanprot_error_accessdenied = -6,
    eMomanprot_error_init = -7,
    eMomanprot_noData = -8
  );
  eMomancmd = (
    //Device Control:
    eMomancmd_shutdown = 16,
    eMomancmd_switchon = 17,
    eMomancmd_disable = 18,
    eMomancmd_quickstop = 19,
    eMomancmd_DiOp = 20,
    eMomancmd_EnOp = 21,
    eMomancmd_faultreset = 22
  );
  eDecoded = (
    eDecoded_none = 0,
    eDecoded_SOBJ = 1,
    eDecoded_GOBJ = 2,
    eDecoded_Bootup = 3,
    eDecoded_NMT = 4,
    eDecoded_NMTRequest = 5,
    eDecoded_Heartbeat = 6,
    eDecoded_Statusword = 7
  );

  //From MomanLib imported functions
  function mmProtInitInterface(InterfaceDll: PAnsiChar;
                               DataReceived: tdmmProtDataCallback;
                               TraceValuesReceived: tdmmProtTraceValuesCallback)
    : eMomanprot; stdcall; external cProtDll;
  procedure mmProtCloseInterface(); stdcall; external cProtDll;
  function mmProtOpenCom(port: Integer; channel: Integer; baud: Integer)
    : eMomanprot; stdcall; external cProtDll;
  procedure mmProtCloseCom; stdcall; external cProtDll;
  function mmProtSendCommand(nodeNr: Integer; index: Integer; subIndex: Integer;
    dataLen: Integer; data: Integer) : Boolean; stdcall; external cProtDll;
  function mmProtReadAnswer(answData: PPAnsiChar;	var nodeNr: Integer; cmdString: PPAnsiChar;
    receiveTelegram: PPAnsiChar) : eMomanprot; stdcall; external cProtDll;
  function mmProtDecodeAnswStr(answStr: PAnsiChar; var value: Int64)
    : eDecoded; stdcall; external cProtDll;
  function mmProtGetStrObj(nodeNr: Integer; index: Integer; subIndex: Integer;
    value: PPAnsiChar) : eMomanprot; stdcall; external cProtDll;
  function mmProtSetObj(nodeNr: Integer; index: Integer; subIndex: Integer; value: Integer;
    len: Integer; var abortCode: Integer)  : eMomanprot; stdcall; external cProtDll;
  function mmProtGetAbortMessage(abortCode: Word)
    : PAnsiChar; stdcall; external cProtDll;

type
  //Sample class:
  TMomanLibSample = class
  private
	  m_isComOpen: Boolean;
	  m_Statusword: Integer;
	  m_abortMessage: String;
  public
    constructor Create;
    destructor Destroy; override;
  	function Init(SignalDataReceived: tdmmProtDataCallback): Boolean;
    function GetStrObj(nodeNr: Integer; index: Integer; subIndex: Integer; var value: String): Boolean;
    function SendCommand(nodeNr: Integer; cmd: eMomancmd): Boolean;
    function ReadReceivedData(var data: String; var cmd: String): Boolean;
    function SetObj(nodeNr: Integer; index: Integer; subIndex: Integer; value: Integer; len: Integer): Boolean;
    function GetStatusword(): Integer;
    function GetAbortMessage(): String;
  end;

//---------------------------------------------------------------------------
implementation

//***************
//*** Public: ***
//***************
constructor TMomanLibSample.Create;
begin
  m_isComOpen := false;
  m_Statusword := 0;
  m_abortMessage := '';
end;

destructor TMomanLibSample.Destroy;
begin
  if m_isComOpen then begin
    mmProtCloseCom();
    mmProtCloseInterface;
  end;
end;

{**
* Initialize the communication Library
*
* \param[in]		SignalDataReceived	callback function to signal received data
* \return		true = initialization succesfull
*						false = error loading the library
*}
function TMomanLibSample.Init(SignalDataReceived: tdmmProtDataCallback): Boolean;
var
  ret: eMomanprot;
begin
  m_isComOpen := false;
  ret := mmProtInitInterface(PAnsiChar(AnsiString(cIntfDll)), SignalDataReceived, NIL);
  if ret <> eMomanprot_ok then
  begin
    result := false;
    exit;
  end;
  if mmProtOpenCom(1, 0, 0) <> eMomanprot_ok then
  begin
    result := false;
    exit;
  end;
  m_isComOpen := true;
  result := true;
end;

{**
* Read data of a string object in the object dictionary
*
* \param[in]		nodeNr		node number
* \param[in]		index			object index
* \param[in]		subIndex		object subindex
* \param[out]		value			object data
* \return		true = object successfully read
*						false = error reading object value
*}
function TMomanLibSample.GetStrObj(nodeNr: Integer; index: Integer; subIndex: Integer; var value: String): Boolean;
var
  ret: eMomanprot;
  answData: PAnsiChar;
begin
  answData := nil;
  ret :=  mmProtGetStrObj(nodeNr, index, subIndex, @answData);
  if (ret = eMomanprot_ok) and (answData <> nil) then begin
    value := String(answData);
    result := true;
  end
  else
    result := false;
end;

{**
* Send a Motion Manager command
*
* \param[in]		nodeNr		node number
* \param[in]		cmd			Motion Manager command
* \return		true = command successfully sent
*						false = error sending command
*}
function TMomanLibSample.SendCommand(nodeNr: Integer; cmd: eMomancmd): Boolean;
begin
  result := mmProtSendCommand(nodeNr, $0000, Integer(cmd), 0, 0);
end;

{**
* Read asynchronous received data
*
* \param[out]		data		received data
* \param[out]		cmd		command assigned to the data
* \return		true = data received
*						false = no data received
*}
function TMomanLibSample.ReadReceivedData(var data: String; var cmd: String): Boolean;
var
  ret: eMomanprot;
  answData: PAnsiChar;
  cmdString: PAnsiChar;
  receiveTelegram: PAnsiChar;
  nodeNr: Integer;
  value: Int64;
begin
  ret := mmProtReadAnswer(@answData, nodeNr, @cmdString, @receiveTelegram);
  if ret <> eMomanprot_noData then
  begin
    data := String(answData);
    cmd := String(cmdString);
    if mmProtDecodeAnswStr(answData, value) = eDecoded_Statusword then
      m_Statusword := value;
    result := true;
  end
  else
    result := false;
end;

{**
* Write data to an object in the object dictionary
*
* \param[in]		nodeNr		node number
* \param[in]		index			object index
* \param[in]		subIndex		object subindex
* \param[in]		value			object data
* \param[in]		len			len of object data
* \return		true = object successfully written
*						false = error writing object value
*								  (an abort message is written to a member variable)
*}
function TMomanLibSample.SetObj(nodeNr: Integer; index: Integer; subIndex: Integer;
  value: Integer; len: Integer): Boolean;
var
  ret: eMomanprot;
  abortCode: Integer;
begin
	ret := mmProtSetObj(nodeNr, index, subIndex, value, len, abortCode);
	if ret = eMomanprot_ok then
  begin
		result := true;
  end
	else
  begin
		m_abortMessage := String(mmProtGetAbortMessage(abortCode));
		result := false;
	end;
end;

{**
* Read last value of the statusword
*
* \return			value of statusword set in ReadReceivedData()
*}
function TMomanLibSample.GetStatusword(): Integer;
begin
	result := m_Statusword;
end;

{**
* Read last abort message
*
* \return			abort message set in SetObj()
*}
function TMomanLibSample.GetAbortMessage(): String;
begin
	result := m_abortMessage;
end;

//---------------------------------------------------------------------------
end.

